function chart_ingreso_egreso (widget) {
    "use strict";

    var model = openerp;

    widget.ChartIngresoEgresoWidget = widget.Base.extend({
        template: 'ChartIngresoEgreso',
        data: [],

        events: {
            'click .week': 'showWeeks',
            'click .days': 'showDays',
            'click .month': 'showMonths',
        },

        init: function (parent) {
            this._super(parent, {
                width: 6,
                height: 4
            });
        }, 
        start: function () {
            var self = this;
            self.fetchInitial();
        },
        fetchInitial:function() {
            var self = this;
            self.$el.block({
                message: null,
                overlayCSS: {
                    backgroundColor: '#FAFAFA'
                }
            });

            self.$el.find('.widget-content.widget-loading').css('display','flex');

            self.fetchCurrentUser().then(function (CurrentUser) {
                return CurrentUser;
            }).then(function (CurrentUser) {
                self.CurrentUser = CurrentUser;
                return self.fetchResUser(CurrentUser);
            }).then(function (ResUser) {
                self.ResUser = ResUser;
                return self.fetchAccountJournal();
            }).then(function (AccountJournal) {
                self.AccountJournal = AccountJournal;
                return self.fecthAccountBankStatementLine();
            }).then(function (AccountBankStatementLine) {
                self.AccountBankStatementLine = AccountBankStatementLine;
                return self.showMonths();
            });
        },

        fetchCurrentUser: function() {
            var self = this;
            var ResUser = new model.web.Model('res.users');
            return ResUser.call('get_user', {
                context: new model.web.CompoundContext()
            });
        },

        fetchResUser: function(id) {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name','store_id'];
            var domain = [['id','=',id]];
            var ResUser = new model.web.Model('res.users');
            ResUser.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },

        fetchAccountJournal: function() {
            var self = this;
            var defer = $.Deferred();
            var store_ids = self.ResUser[0].store_id[0];
            var fields = ['id', 'name', 'store_ids'];
            var domain = [['type','in',['bank','cash']],['store_ids','in',store_ids]];
            var AccountJournal = new model.web.Model('account.journal');
            AccountJournal.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });

            return defer;
        },

        fecthAccountBankStatementLine: function() {
            var self = this;
            var defer = $.Deferred();
            var journal_ids = _.flatten(_.map(self.AccountJournal, function (item) {
                return item.id;
            }));
            var fields = ['id','date','amount'];
            var domain = [['journal_id','in',journal_ids]];
            var AccountBankStatementLine = new model.web.Model('account.bank.statement.line');
            AccountBankStatementLine.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });
            return defer;
        },

        // Obtener Lineas de caja por mes
        
        getInAccountBankStatementLine:function(mes) {
            var self = this;
            if (mes < 10){
                var fecha = moment().format('YYYY')+'-'+'0'+mes;    
            }else{
                var fecha = moment().format('YYYY')+'-'+mes;    
            }
            return _.flatten(_.filter(self.AccountBankStatementLine,function (inv) {
                return moment(inv.date).format('YYYY-MM') === fecha & inv.amount > 0;
            }));
        },

        getOutAccountBankStatementLine:function(mes) {
            var self = this;
            if (mes < 10){
                var fecha = moment().format('YYYY')+'-'+'0'+mes;    
            }else{
                var fecha = moment().format('YYYY')+'-'+mes;    
            }
            return _.flatten(_.filter(self.AccountBankStatementLine,function (inv) {
                return moment(inv.date).format('YYYY-MM') === fecha & inv.amount < 0;
            }));
        },

        // Obtener lineas de caja por esta semana

        getInWeekAccountBankStatementLine:function(day) {
            var self = this;
            var week = moment().week();
            return _.flatten(_.filter(self.AccountBankStatementLine,function (inv) {
                return moment(inv.date).week() === week & moment(inv.date).isoWeekday() === day & inv.amount > 0;
            }));
        },

        getOutWeekAccountBankStatementLine:function(day) {
            var self = this;
            var week = moment().week();
            return _.flatten(_.filter(self.AccountBankStatementLine,function (inv) {
                return moment(inv.date).week() === week & moment(inv.date).isoWeekday() === day & inv.amount < 0;
            }));
        },

        // Obtener lineas de caja por dias

        getInDaysAccountBankStatementLine:function(day) {
            var self = this;
            var date = moment().subtract(day, 'days').format('YYYY-MM-DD');
            return _.flatten(_.filter(self.AccountBankStatementLine,function (inv) {
                return moment(inv.date).format('YYYY-MM-DD') === date & inv.amount > 0;
            }));
        },

        getOutDaysAccountBankStatementLine:function(day) {
            var self = this;
            var date = moment().subtract(day, 'days').format('YYYY-MM-DD');
            return _.flatten(_.filter(self.AccountBankStatementLine,function (inv) {
                return moment(inv.date).format('YYYY-MM-DD') === date & inv.amount < 0; 
            }));
        },
  
        // Mostrar por meses
        showMonths: function() {
            var self = this;
            var title = ['Ene', 'Feb', 'Mar','Abr','May','Jun','Jul','Ago','Sep','Oct','Nov','Dic'];
            var inStatement;
            var OutStatement;
            var mes = 0;
            var dataIn = [];
            var dataOut = [];
            for (var i = 1; i <= 12; i++) {
                inStatement = self.getInAccountBankStatementLine(i);
                var total = _.reduce(_.map(inStatement,function(item) {
                    return item.amount;
                }),function(memo, num) {
                    return memo + num;
                },0);
                dataIn.push(total);
            }
            for (var i = 1; i <= 12; i++) {
                OutStatement = self.getOutAccountBankStatementLine(i);
                var total = _.reduce(_.map(OutStatement,function(item) {
                    return item.amount;
                }),function(memo, num) {
                    return memo + num;
                },0);
                dataOut.push(Math.abs(total));
            }
            self.$el.unblock();
            self.$el.find('.widget-content.widget-loading').css('display','none');
            self.fetchChart(dataIn, dataOut, title);
        },

        // Mostrar por semana Actual
        showWeeks: function() {
            var self = this;
            var title = ['Lunes', 'Martes', 'Miercoles','Jueves','Viernes','Sabado','Domingo'];
            var inStatement;
            var OutStatement;
            var mes = 0;
            var dataIn = [];
            var dataOut = [];
            for (var i = 1; i <= 7; i++) {
                inStatement = self.getInWeekAccountBankStatementLine(i);
                var total = _.reduce(_.map(inStatement,function(item) {
                    return item.amount;
                }),function(memo, num) {
                    return memo + num;
                },0);
                dataIn.push(total);
            }
            for (var i = 1; i <= 7; i++) {
                OutStatement = self.getOutWeekAccountBankStatementLine(i);
                var total = _.reduce(_.map(OutStatement,function(item) {
                    return item.amount;
                }),function(memo, num) {
                    return memo + num;
                },0);
                dataOut.push(Math.abs(total));
            }
            self.$el.unblock();
            self.$el.find('.widget-content.widget-loading').css('display','none');
            self.fetchChart(dataIn, dataOut, title);
        },

        // Mostrar grafico por dias
        showDays: function() {
            var self = this;
            var title = [];
            var inStatement;
            var OutStatement;
            var mes = 0;
            var dataIn = [];
            var dataOut = [];
            for (var i = 15; i >= 0; i--) {
                inStatement = self.getInDaysAccountBankStatementLine(i);
                var total = _.reduce(_.map(inStatement,function(item) {
                    return item.amount;
                }),function(memo, num) {
                    return memo + num;
                },0);
                dataIn.push(total);
                title.push(moment().subtract(i, 'days').format('DD / MM'));
            }
            for (var i = 15; i >= 0; i--) {
                OutStatement = self.getOutDaysAccountBankStatementLine(i);
                var total = _.reduce(_.map(OutStatement,function(item) {
                    return item.amount;
                }),function(memo, num) {
                    return memo + num;
                },0);
                dataOut.push(Math.abs(total));
            }
            self.$el.unblock();
            self.$el.find('.widget-content.widget-loading').css('display','none');
            self.fetchChart(dataIn, dataOut, title);
        },

        // Generar Grafico
        fetchChart: function (dataIn,dataOut,title) {
            var self = this;
            var label = title;
            var bodyIn = dataIn;
            var bodyOut = dataOut;
            var item;

            // Global method for setting Y axis number format.
            Chart.scaleService.updateScaleDefaults('linear', {
              ticks: {
                callback: function(tick) {
                    return tick.toLocaleString('de-DE');
                }
              }
            }); 
            Chart.defaults.global.tooltips.callbacks.label = function(tooltipItem, data) {
                var dataset = data.datasets[tooltipItem.datasetIndex];
                var datasetLabel = dataset.label || '';
                return datasetLabel +  dataset.data[tooltipItem.index].toLocaleString('de-DE');
            };

            var chart = new Chart(this.$el.find(".widget-content").find('canvas'), {
                type: 'line',
                data: {
                    labels: label,
                    datasets: [
                        {
                            label: 'Ingreso  ',
                            data: bodyIn,
                            backgroundColor: 'rgba(51, 141, 255, 0.2)',
                            borderColor: '#0288d1',
                            borderWidth: 2,
                            fill: false,
                        },
                        {
                            label: 'Egreso  ',                       
                            data: bodyOut,
                            backgroundColor: 'rgba(255, 110, 59, 0.2)',
                            borderColor: '#ef6c00',
                            borderWidth: 2,
                            fill: false,
                        }
                    ]
                },
                options: {
                    responsive: true,
                    legend: {
                        display: false,
                    },
                    title: {
                        display: false,
                    },
                    layout: {
                        padding: {
                            top: 0,
                            bottom: 45,
                            left : 0,
                            rigth: 0,
                        }
                    },
                    events: ['click'],
                    elements: {
                        line: {
                            tension: 0.000001
                        },
                    },
                }
            });
        },
    });
}
