function chart_sale_order_product (widget) {
    "use strict";

    var model = openerp;

    widget.ChartSaleOrderProductWidget = widget.Base.extend({
        template: 'ChartSaleOrderProduct',
        data: [],

        events: {
            
        },
        init: function (parent) {
            this._super(parent, {
                width: 6,
                height: 4
            });
        },
        start: function () {
            var self = this;
            self.fetchInitial();
        },
        fetchInitial: function() {
            var self = this;
            self.fetchCurrentUser().then(function (CurrentUser) {
                return CurrentUser;
            }).then(function (CurrentUser) {
                self.CurrentUser = CurrentUser;
                return self.fetchResUser(CurrentUser);
            }).then(function(ResUser) {
                self.ResUser = ResUser;
                return self.fetchAccountJournal();
            }).then(function(AccountJournal) {
                self.AccountJournal = AccountJournal;
                return self.fetchAccountInvoice();
            }).then(function (AccountInvoice) {
                self.AccountInvoice = AccountInvoice;
                return self.fetchAccountInvoiceLine();
            }).then(function (AccountInvoiceLine) {
                self.AccountInvoiceLine = AccountInvoiceLine;
                return self.fecthProductProduct();
            }).then(function(ProductProduct){
                self.ProductProduct = ProductProduct;
                return self.fetchProductRanking();
            });
        },

        /* 
            USER
        */

        fetchCurrentUser: function() {
            var self = this;
            var ResUser = new model.web.Model('res.users');
            return ResUser.call('get_user', {
                context: new model.web.CompoundContext()
            });
        },

        fetchResUser: function(id) {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name','store_id'];
            var domain = [['id','=',id]];
            var ResUser = new model.web.Model('res.users');
            ResUser.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },

        fetchAccountJournal: function() {
            var self = this;
            var defer = $.Deferred();
            var store_ids = self.ResUser[0].store_id[0];
            var fields = ['id', 'name', 'store_ids'];
            var domain = [['type','=','sale'],['store_ids','in',store_ids]];
            var AccountJournal = new model.web.Model('account.journal');
            AccountJournal.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });
            return defer;
        },

        fetchAccountInvoice: function() {
            var self = this;
            var defer = $.Deferred();
            var journal_ids = _.flatten(_.map(self.AccountJournal, function (item) {
                return item.id;
            }));
            var desde = moment().format('YYYY-MM-01');
            var hasta = moment().add(1,'months').format('YYYY-MM-01');
            var fields = ['id', 'invoice_line', 'date_order'];
            var domain = [['date_invoice', '>=', desde], ['date_invoice', '<', hasta], ['state', 'in', ['open','paid']],['journal_id','in',journal_ids],['type','=','out_invoice']];
            var AccountInvoice = new model.web.Model('account.invoice');
            AccountInvoice.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });
            return defer;
        },

        fetchAccountInvoiceLine: function() {
            var self = this;
            var defer = $.Deferred();
            var invoice_line = _.flatten(_.map(self.AccountInvoice, function (item) {
                return item.invoice_line;
            }));
            var fields = ['id', 'product_id', 'quantity'];
            var domain = [['id','in', invoice_line]];
            var AccountInvoiceLine = new model.web.Model('account.invoice.line');
            AccountInvoiceLine.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });
            return defer;
        },

        fecthProductProduct: function () {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id', 'name_template','attribute_str'];
            var domain = [['active', '=', true]];
            var ProductProduct = new model.web.Model('product.product');
            ProductProduct.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },

        getAccountInvoiceLine: function (id) {
            var self = this;
            return _.flatten(_.filter(self.AccountInvoiceLine,function (item) {
                return item.product_id[0] === id;
            }));
        },
        
        fetchProductRanking: function() {
            var self = this;
            var AccountInvoiceLine;
            var ranking = [];
            var quantity = 0;
            var attribute;
            for (var i = 0; i < self.ProductProduct.length; i++) {
                AccountInvoiceLine = self.getAccountInvoiceLine(self.ProductProduct[i].id);
                if (AccountInvoiceLine.length > 0) {
                    var invoice_quantity = _.reduce(_.map(AccountInvoiceLine, function (map) {
                        return map.quantity
                    }), function (meno,num) {
                        return meno + num
                    }, 0);
                    if(self.ProductProduct[i].attribute_str){
                        attribute = '*' + self.ProductProduct[i].attribute_str;
                    }else{
                        attribute = ' ';
                    }
                    ranking.push({
                        id: self.ProductProduct[i].id,
                        product: self.ProductProduct[i].name_template + ' ' + attribute,
                        qty: invoice_quantity,
                    });
                }
            }
            ranking.sort(function(a, b) {
                return b.qty - a.qty
            });
            self.fetchChart(ranking);
        },     


        fetchChart: function (ranking){
            var self = this;
            var label = [];
            var body = [];
            var rank = 7;
            var item;

            for (var i = 0; i < rank; i++) {
                if (ranking[i]){
                    item = ranking[i];
                }else{
                    item = {};
                    item.product = "N/A";
                    item.qty = 0;
                }
                var name = item.product.split('*');
                if (name.length === 1) {
                        label.push(name[0]);
                }else{
                    label.push(name[0]+"("+name[name.length-1]+")");
                }
                body.push(item.qty);
            }

            var chart = new Chart(this.$el.find(".widget-content").find('canvas'), {
                type: 'horizontalBar',
                data: {
                    labels: label,
                    datasets: [
                        {
                            label: 'Productos ',
                            borderWidth: 2,
                            backgroundColor: '#e3f2fd',
                            borderColor: '#64b5f6',
                            data: body,
                            fill: true,
                        },
                    ]
                },
                options: {
                    responsive: true,
                    title: {
                        display: false,
                    },
                    hover: {
                        mode: 'nearest',
                        intersect: true
                    },
                    legend: {
                       display: false,
                    },
                    layout: {
                        padding: {
                            top: 0,
                            bottom: 40,
                            left: 0,
                            rigth: 0,
                        }
                    },
                    events: ['click'],
                }
            });
        },
    });
}
