function chart_sale_order_salesman (widget) {
    "use strict";

    var model = openerp;

    widget.ChartSaleOrderSalesmanWidget = widget.Base.extend({
        template: 'ChartSaleOrderSalesman',
        events: {

        },

        init: function (parent) {
            this._super(parent, {
                width: 6,
                height: 4
            });
        },
        start: function () {
            var self = this;
            self.fetchInitial();
        },
        fetchInitial:function() {
            var self = this;
            self.fetchCurrentUser().then(function (CurrentUser) {
                return CurrentUser;
            }).then(function (CurrentUser) {
                self.CurrentUser = CurrentUser;
                return self.fetchResUser();
            }).then(function(ResUser) {
                self.ResUser = ResUser;
                return self.fetchAccountJournal(self.CurrentUser);
            }).then(function(AccountJournal) {
                self.AccountJournal = AccountJournal;
                return self.fetchAccountInvoice();
            }).then(function (AccountInvoice) {
                self.AccountInvoice = AccountInvoice;
                return self.fetchResCompany(self.CurrentUser);
            }).then(function(ResCompany) {
                self.ResCompany = ResCompany;
                return self.fetchResCurrency();
            }).then(function(ResCurrency) {
                self.ResCurrency = ResCurrency;
                return self.fetchProductRanking();
            });
        },

        /* 
            USER
        */

        fetchCurrentUser: function() {
            var self = this;
            var ResUser = new model.web.Model('res.users');
            return ResUser.call('get_user', {
                context: new model.web.CompoundContext()
            });
        },

        fetchResUser: function() {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name','store_id','company_id'];
            var domain = [['active','=',true]];
            var ResUser = new model.web.Model('res.users');
            ResUser.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },

        fetchAccountJournal: function(id) {
            var self = this;
            var defer = $.Deferred();
            var user = _.flatten(_.filter(self.ResUser,function (inv) {
                return inv.id === id;
            }));
            var store_ids = user[0].store_id[0];
            var fields = ['id', 'name', 'store_ids'];
            var domain = [['type','=','sale'],['store_ids','in',store_ids]];
            var AccountJournal = new model.web.Model('account.journal');
            AccountJournal.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });
            return defer;
        },

        fetchAccountInvoice: function() {
            var self = this;
            var defer = $.Deferred();
            var journal_ids = _.flatten(_.map(self.AccountJournal, function (item) {
                return item.id;
            }));
            var desde = moment().format('YYYY-MM-01');
            var hasta = moment().add(1,'months').format('YYYY-MM-01');
            var fields = ['id', 'invoice_line', 'date_order','user_id','amount_total','currency_id'];
            var domain = [['date_invoice', '>=', desde], ['date_invoice', '<', hasta], ['state', 'in', ['open','paid']],['journal_id','in',journal_ids],['type','=','out_invoice']];
            var AccountInvoice = new model.web.Model('account.invoice');
            AccountInvoice.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });
            return defer;
        },

        fetchResCompany: function(id) {
            var self = this;
            var defer = $.Deferred();
            var user = _.flatten(_.filter(self.ResUser,function (inv) {
                return inv.id === id;
            }));
            var fields = ['id','name', 'currency_id'];
            var domain = [['id', '=', user[0].company_id[0]]];
            var ResCompany = new model.web.Model('res.company');
            ResCompany.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });
            return defer;
        },

        fetchResCurrency : function(){
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name', 'symbol', 'rate_silent', 'base', 'decimal_separator', 'decimal_places', 'thousands_separator', 'symbol_position'];
            var domain = [['active', '=', true]];
            var ResCurrency = new model.web.Model('res.currency');
            ResCurrency.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });
            return defer;
        },

        getResCurrency: function (id) {
            var self = this;
            return _.filter(self.ResCurrency,function (item) {
                return item.id === id;
            })
        },

        getAccountInvoice:function(id) {
            var self = this;
            return _.flatten(_.filter(self.AccountInvoice,function (inv) {
                return inv.user_id[0] === id;
            }));
        },

        fetchProductRanking: function() {
            var self = this;
            var AccountInvoice;
            var ranking = [];
            var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
            for (var i = 0; i < self.ResUser.length; i++) {
                var data = [];
                var amount = 0;
                AccountInvoice = self.getAccountInvoice(self.ResUser[i].id);
                if (AccountInvoice.length > 0) {
                    _.each(AccountInvoice, function (item) {
                        var currency = self.getResCurrency(item.currency_id[0]).shift();
                        data.push({
                            amount: item.amount_total * (CurrencyBase.rate_silent / currency.rate_silent),
                        })
                    });
                    if(data.length > 0){
                        amount = _.reduce(_.map(data, function (map) {
                            return map.amount;
                        }), function (memo, num) {
                            return memo + num;
                        }); 
                    }
                    ranking.push({
                        id: self.ResUser[i].id,
                        name: self.ResUser[i].name,
                        amount: accounting.formatNumber(amount,0,"",","),
                    });
                }
            }
            ranking.sort(function (a, b) {
                return b.amount - a.amount
            });
            console.log(ranking);
            self.fetchChart(ranking);
        },

        fetchChart: function (ranking) {
            var self = this;
            var label = [];
            var body = [];
            var item;
            var rank = 7;

            if (ranking.length < rank && ranking.length > 0 )
                rank= ranking.length;

            for (var i = 0; i < rank; i++) {
                if (ranking[i]) {
                    item = ranking[i];
                }
                if (ranking.length === 0) {
                    item = {};
                    item.name = "N/A";
                    item.amount = 0;
                }
                label.push(item.name);
                body.push(item.amount);
            }
            // Global method for setting Y axis number format.
            Chart.scaleService.updateScaleDefaults('linear', {
              ticks: {
                callback: function(tick) {
                    return tick.toLocaleString('de-DE');
                }
              }
            });
            Chart.defaults.global.tooltips.callbacks.label = function(tooltipItem, data) {
                var dataset = data.datasets[tooltipItem.datasetIndex];
                var datasetLabel = dataset.label || '';
                return datasetLabel +  dataset.data[tooltipItem.index].toLocaleString('de-DE');
            };
            var chart = new Chart(this.$el.find(".widget-content").find('canvas'), {
                type: 'horizontalBar',
                data: {
                    labels: label,
                    datasets: [
                        {
                            label: 'Ventas  ',
                            backgroundColor: '#e3f2fd',
                            borderColor: '#64b5f6',
                            borderWidth: 1,
                            data: body,
                        }
                    ]
                },
                options: {
                    responsive: true,
                    title: {
                        display: false,
                    },
                    hover: {
                        mode: 'nearest',
                        intersect: true
                    },
                    legend: {
                       display: false,
                    },
                    layout: {
                        padding: {
                            top: 0,
                            bottom: 40,
                            left: 0,
                            rigth: 0,
                        }
                    },
                    events: ['click'],
                }
            });
        },
    });
}
