function widget_balance(widget) {
    "use strict";

    var model = openerp;
    var Qweb = openerp.web.qweb;

    widget.WidgetBalanceWidget = widget.Base.extend({
        template: 'WidgetBalance',
        month: 0,
        week: 0,
        today: 0,
        data: 0,
        objective: 0,
        percentage: 0, 

        events: {
           'click .number': 'showModal',
        },

        init: function (parent) {
            this._super(parent, {
                width: 3,
                height: 2
            });
        },

        start: function () {
            var self = this;
            self.fetchInitial();
        },

        fetchInitial: function(){
            var self = this;
            self.fetchCurrentUser().then(function (CurrentUser) {
                return CurrentUser;
            }).then(function (CurrentUser) {
                self.CurrentUser = CurrentUser;
                return self.fetchResUser(CurrentUser);
            }).then(function(ResUser) {
                self.ResUser = ResUser;
                return self.fetchAccountJournal();
            }).then(function(AccountJournal) {
                self.AccountJournal = AccountJournal;
                return self.fetchAccountInvoice();
            }).then(function(AccountInvoice) {
                self.AccountInvoice = AccountInvoice;
                return self.fetchPosOrder();
            }).then(function(PosOrder) {
                self.PosOrder = PosOrder;
                return self.fetchResCompany();
            }).then(function(ResCompany) {
                self.ResCompany = ResCompany;
                return self.fetchResCurrency();
            }).then(function(ResCurrency) {
                self.ResCurrency = ResCurrency;
                return self.fetchDashboardObjetive();
            }).then(function(DashboardObjetive) {
                self.DashboardObjetive = DashboardObjetive;
                return self.showThisMonth();
            });
        },

        /* 
            USER
        */

        fetchCurrentUser: function() {
            var self = this;
            var ResUser = new model.web.Model('res.users');
            return ResUser.call('get_user', {
                context: new model.web.CompoundContext()
            });
        },

        fetchResUser: function(id) {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name','store_id'];
            var domain = [['id','=',id]];
            var ResUser = new model.web.Model('res.users');
            ResUser.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },

        fetchAccountJournal: function() {
            var self = this;
            var defer = $.Deferred();
            var store_ids = self.ResUser[0].store_id[0];
            var fields = ['id', 'name', 'store_ids'];
            var domain = [['type','in',['sale','purchase']],['store_ids','in',store_ids]];
            var AccountJournal = new model.web.Model('account.journal');
            AccountJournal.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });
            return defer;
        },

        fetchAccountInvoice: function() {
            var self = this;
            var defer = $.Deferred();
            var journal_ids = _.flatten(_.map(this.AccountJournal, function (item) {
                return item.id;
            }));
            var fields = ['id','type','date_invoice','amount_total','currency_id'];
            var domain = [['state', 'not in', ['draft','cancel']],['type','in',['in_invoice','out_invoice']],['journal_id','in',journal_ids]];
            var AccountInvoice = new model.web.Model('account.invoice');
            AccountInvoice.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });
            return defer;
        },

        fetchPosOrder: function() {
            var self = this;
            var defer = $.Deferred();
            var journal_ids = _.flatten(_.map(this.AccountJournal, function (item) {
                return item.id;
            }));
            var fields = ['id', 'name', 'date_order', 'amount_total','sale_journal'];
            var domain = [['state', 'not in', ['draft','cancel']],['sale_journal','in',journal_ids]];
            var PosOrder = new model.web.Model('pos.order');
            PosOrder.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });

            return defer;
        },

        fetchResCompany: function() {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name', 'currency_id'];
            var domain = [['id', '=', 1]];
            var ResCompany = new model.web.Model('res.company');
            ResCompany.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },

        fetchResCurrency : function(){
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name', 'symbol', 'rate_silent', 'base', 'decimal_separator', 'decimal_places', 'thousands_separator', 'symbol_position'];
            var domain = [['active', '=', true]];
            var ResCurrency = new model.web.Model('res.currency');
            ResCurrency.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });
            return defer;
        },

        fetchDashboardObjetive : function(){
            var self = this;
            var defer = $.Deferred();
            var store_id = self.ResUser[0].store_id[0];
            var date_min = moment().format('YYYY-MM-01');
            var date_max = moment().add(1, 'month').format('YYYY-MM-01');
            var fields = ['id','store_id', 'date', 'sale_objetive', 'purchase_limit', 'expense_limit', 'expected_profit'];
            var domain = [['date', '>=', date_min],['date', '<', date_max],['store_id','=',store_id]];
            var DashboardObjetive = new model.web.Model('dashboard.objetive');
            DashboardObjetive.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });
            return defer;
        },

        getResCurrency: function (id) {
            var self = this;
            return _.filter(self.ResCurrency,function (item) {
                return item.id === id;
            })
        },

        /* 
            ACCOUNT INVOICE - TODAY
        */

        getTodayInAccountInvoice:function() {
            var self = this;
            var date = moment().format('YYYY-MM-DD');
            return _.flatten(_.filter(self.AccountInvoice,function (inv) {
                return moment(inv.date_invoice).format('YYYY-MM-DD') === date & inv.type == 'in_invoice';
            }));
        },

        getTodayOutAccountInvoice:function() {
            var self = this;
            var date = moment().format('YYYY-MM-DD');
            return _.flatten(_.filter(self.AccountInvoice,function (inv) {
                return moment(inv.date_invoice).format('YYYY-MM-DD') === date & inv.type == 'out_invoice';
            }));
        },

        /* 
            ACCOUNT INVOICE - WEEK
        */

        getThisWeekInAccountInvoice:function() {
            var self = this;
            var week = moment().week();
            return _.flatten(_.filter(self.AccountInvoice,function (inv) {
                return moment(inv.date_invoice).week() === week & moment(inv.date_invoice).format('YYYY')=== moment().format('YYYY') & inv.type == 'in_invoice';
            }));
        },

        getThisWeekOutAccountInvoice:function() {
            var self = this;
            var week = moment().week();
            return _.flatten(_.filter(self.AccountInvoice,function (inv) {
                return moment(inv.date_invoice).week() === week & moment(inv.date_invoice).format('YYYY')=== moment().format('YYYY') & inv.type == 'out_invoice';
            }));
        },

        /* 
            ACCOUNT INVOICE - MONTH
        */

        getThisMonthInAccountInvoice:function() {
            var self = this;
            return _.flatten(_.filter(self.AccountInvoice,function (inv) {
                return moment(inv.date_invoice).format('YYYY-MM')=== moment().format('YYYY-MM') & inv.type == 'in_invoice';
            }));
        },

        getThisMonthOutAccountInvoice:function() {
            var self = this;
            return _.flatten(_.filter(self.AccountInvoice,function (inv) {
                return moment(inv.date_invoice).format('YYYY-MM')=== moment().format('YYYY-MM') & inv.type == 'out_invoice';
            }));
        },

        /* 
            POS ORDER
        */
        
        getTodayPosOrder:function() {
            var self = this;
            var date = moment().format('YYYY-MM-DD');
            var tz = moment.tz.guess();
            return _.flatten(_.filter(self.PosOrder,function (inv) {
                var utc = moment.utc(inv.date_order,'YYYY-MM-DD h:mm:ss A');
                return utc.clone().tz(tz).format("YYYY-MM-DD") === date;
            }));
        },

        getThisWeekPosOrder:function() {
            var self = this;
            var week = moment().week();
            var tz = moment.tz.guess();
            return _.flatten(_.filter(self.PosOrder,function (inv) {
                var utc = moment.utc(inv.date_order,'YYYY-MM-DD h:mm:ss A');
                utc = utc.clone().tz(tz).format("YYYY-MM-DD"); 
                return moment(utc).week() === week & moment(utc).format('YYYY')=== moment().format('YYYY');
            }));
        },

        getThisMonthPosOrder:function() {
            var self = this;
            var tz = moment.tz.guess();
            return _.flatten(_.filter(self.PosOrder,function (inv) {
                var utc = moment.utc(inv.date_order,'YYYY-MM-DD h:mm:ss A');
                return utc.clone().tz(tz).format("YYYY-MM") === moment().format('YYYY-MM');
            }));
        }, 

        /* 
            TODAY
        */

        showToday: function () {
            var self = this;
            var amount_order = 0;
            var invoice_in_amount = 0;
            var invoice_out_amount = 0;
            var balance = 0;
            var in_array = [];
            var out_array = [];
            var order = self.getTodayPosOrder();
            var in_invoice = self.getTodayInAccountInvoice();
            var out_invoice = self.getTodayOutAccountInvoice();
            var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
            if(order.length > 0){
                amount_order = _.reduce(_.map(order, function (map) {
                    return map.amount_total;
                }), function (memo, num) {
                    return memo + num;
                });
            }
            
            if(in_invoice.length > 0){
                _.each(in_invoice, function (item) {
                    var currency = self.getResCurrency(item.currency_id[0]).shift();
                    in_array.push({
                        amount: item.amount_total * (CurrencyBase.rate_silent / currency.rate_silent),
                    })
                });
                invoice_in_amount = _.reduce(_.map(in_array, function (map) {
                    return map.amount;
                }), function (memo, num) {
                    return memo + num;
                });
            }
            
            if(out_invoice.length > 0){
                _.each(out_invoice, function (item) {
                    var currency = self.getResCurrency(item.currency_id[0]).shift();
                    out_array.push({
                        amount: item.amount_total * (CurrencyBase.rate_silent / currency.rate_silent),
                    })
                });
                invoice_out_amount = _.reduce(_.map(out_array, function (map) {
                    return map.amount;
                }), function (memo, num) {
                    return memo + num;
                });
            }
            
            balance = (invoice_out_amount + amount_order) - invoice_in_amount;
            
            self.today = balance;
        },

        /* 
            WEEK
        */

        showThisWeek: function () {
            var self = this;
            var amount_order = 0;
            var invoice_in_amount = 0;
            var invoice_out_amount = 0;
            var balance = 0;
            var in_array = [];
            var out_array = [];
            var order = self.getThisWeekPosOrder();
            var in_invoice = self.getThisWeekInAccountInvoice();
            var out_invoice = self.getThisWeekOutAccountInvoice();
            var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
            if(order.length > 0){
                amount_order = _.reduce(_.map(order, function (map) {
                    return map.amount_total;
                }), function (memo, num) {
                    return memo + num;
                });
            }

            if(in_invoice.length > 0){
                _.each(in_invoice, function (item) {
                    var currency = self.getResCurrency(item.currency_id[0]).shift();
                    in_array.push({
                        amount: item.amount_total * (CurrencyBase.rate_silent / currency.rate_silent),
                    })
                });
                invoice_in_amount = _.reduce(_.map(in_array, function (map) {
                    return map.amount;
                }), function (memo, num) {
                    return memo + num;
                });
            }
            
            if(out_invoice.length > 0){
                _.each(out_invoice, function (item) {
                    var currency = self.getResCurrency(item.currency_id[0]).shift();
                    out_array.push({
                        amount: item.amount_total * (CurrencyBase.rate_silent / currency.rate_silent),
                    })
                });
                invoice_out_amount = _.reduce(_.map(out_array, function (map) {
                    return map.amount;
                }), function (memo, num) {
                    return memo + num;
                });
            }

            balance = (invoice_out_amount + amount_order) - invoice_in_amount;
            self.week = balance;
        },

        /* 
            MONTH
        */

        showThisMonth: function () {
            var self = this;

            if(self.DashboardObjetive.length > 0){
                self.objective = self.DashboardObjetive.shift().expected_profit;   
            }else{
                self.objective = 0;    
            }

            var amount_order = 0;
            var invoice_in_amount = 0;
            var invoice_out_amount = 0;
            var balance = 0;
            var in_array = [];
            var out_array = [];
            var order = self.getThisMonthPosOrder();
            var in_invoice = self.getThisMonthInAccountInvoice();
            var out_invoice = self.getThisMonthOutAccountInvoice();
            var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
            
            if(order.length > 0){
                amount_order = _.reduce(_.map(order, function (map) {
                    return map.amount_total;
                }), function (memo, num) {
                    return memo + num;
                });
            }
            
            if(in_invoice.length > 0){
                _.each(in_invoice, function (item) {
                    var currency = self.getResCurrency(item.currency_id[0]).shift();
                    in_array.push({
                        amount: item.amount_total * (CurrencyBase.rate_silent / currency.rate_silent),
                    })
                });
                invoice_in_amount = _.reduce(_.map(in_array, function (map) {
                    return map.amount;
                }), function (memo, num) {
                    return memo + num;
                });
            }
            
            if(out_invoice.length > 0){
                _.each(out_invoice, function (item) {
                    var currency = self.getResCurrency(item.currency_id[0]).shift();
                    out_array.push({
                        amount: item.amount_total * (CurrencyBase.rate_silent / currency.rate_silent),
                    })
                });
                invoice_out_amount = _.reduce(_.map(out_array, function (map) {
                    return map.amount;
                }), function (memo, num) {
                    return memo + num;
                });
            }

            balance = (invoice_out_amount + amount_order) - invoice_in_amount;           
            self.month = balance;
            if(self.objective > 0){ 
                var percentage = (balance*100)/self.objective;    
            }else{
                var percentage = 0;
            }
            var chart4 = new ldBar("#circle-balance",{
                'stroke':'#0288d1',
                'stroke-width':'10'
            });
            chart4.set(percentage);
            self.percentage = percentage;
            
            self.$el.find('.widget-content').find('a').text(accounting.formatMoney(balance, CurrencyBase.symbol, CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator))
        },

        /* 
            MODAL
        */

        showModal: function (e) {
            var self = this;
            self.showThisWeek();
            self.showToday();
            var titleData = [
                {
                    title: "Balance"
                }
            ];
            var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
            var modal = Qweb.render('WidgetModal', {
                modalTitle: titleData,
                objective: accounting.formatMoney(self.objective, CurrencyBase.symbol, CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),
                today: accounting.formatMoney(self.today, CurrencyBase.symbol, CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),
                week: accounting.formatMoney(self.week, CurrencyBase.symbol, CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),
                month: accounting.formatMoney(self.month, CurrencyBase.symbol, CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),
            });
            $('.openerp_webclient_container').after(modal);
            $('.widget-modal').modal()
            $('.widget-modal').on('hidden.bs.modal', function (e) {
                self.removeModal(e);
            })

            var chartX = new ldBar("#myItemModal",{
                'stroke':'#0288d1',
            });

            chartX.set(self.percentage);
        },

        removeModal: function (e) {
            $('.widget-modal').remove();
            $('.modal-backdrop').remove();
        },

    });
}