function widget_purchase(widget) {
    "use strict";

    var model = openerp;
    var Qweb = openerp.web.qweb;

    widget.WidgetPurchaseWidget = widget.Base.extend({
        template: 'WidgetPurchase',
        month: 0,
        week: 0,
        today: 0,
        data: 0,
        objective: 0,
        percentage: 0,

        events: {
            'click .number': 'showModal',
        }, 

        init: function (parent) {
            this._super(parent, {
                width: 3,
                height: 2
            });
        },

        start: function () {
            var self = this;
            self.fetchInitial();
        },

        fetchInitial: function(){
            var self = this;
            self.fetchCurrentUser().then(function (CurrentUser) {
                return CurrentUser;
            }).then(function (CurrentUser) {
                self.CurrentUser = CurrentUser;
                return self.fetchResUser(CurrentUser);
            }).then(function(ResUser) {
                self.ResUser = ResUser;
                return self.fetchAccountJournal();
            }).then(function(AccountJournal) {
                self.AccountJournal = AccountJournal;
                return self.fetchAccountInvoice();
            }).then(function(AccountInvoice) {
                self.AccountInvoice = AccountInvoice;
                return self.fetchResCompany();
            }).then(function(ResCompany) {
                self.ResCompany = ResCompany;
                return self.fetchResCurrency();
            }).then(function(ResCurrency) {
                self.ResCurrency = ResCurrency;
                return self.fetchDashboardObjetive();
            }).then(function(DashboardObjetive) {
                self.DashboardObjetive = DashboardObjetive;
                return self.showThisMonth();
            });
        },

        /* 
            USER
        */

        fetchCurrentUser: function() {
            var self = this;
            var ResUser = new model.web.Model('res.users');
            return ResUser.call('get_user', {
                context: new model.web.CompoundContext()
            });
        },

        fetchResUser: function(id) {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name','store_id'];
            var domain = [['id','=',id]];
            var ResUser = new model.web.Model('res.users');
            ResUser.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },

        fetchAccountJournal: function() {
            var self = this;
            var defer = $.Deferred();
            var store_ids = self.ResUser[0].store_id[0];
            var fields = ['id', 'name', 'store_ids'];
            var domain = [['type','=','purchase'],['store_ids','in',store_ids]];
            var AccountJournal = new model.web.Model('account.journal');
            AccountJournal.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });

            return defer;
        },

        fetchAccountInvoice: function() {
            var self = this;
            var defer = $.Deferred();
            var journal_ids = _.flatten(_.map(this.AccountJournal, function (item) {
                return item.id;
            }));
            var fields = ['id', 'name', 'date_invoice', 'amount_total','currency_id'];
            var domain = [['state', 'not in', ['draft','cancel']],['origin','like','PO'],['type','=','in_invoice'],['journal_id','in',journal_ids]];
            var AccountInvoice = new model.web.Model('account.invoice');
            AccountInvoice.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });
            return defer;
        },

        fetchResCompany: function() {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name', 'currency_id'];
            var domain = [['id', '=', 1]];
            var ResCompany = new model.web.Model('res.company');
            ResCompany.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });
            return defer;
        },

        fetchResCurrency : function(){
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name', 'symbol', 'rate_silent', 'base', 'decimal_separator', 'decimal_places', 'thousands_separator', 'symbol_position'];
            var domain = [['active', '=', true]];
            var ResCurrency = new model.web.Model('res.currency');
            ResCurrency.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });
            return defer;
        },

        fetchDashboardObjetive : function(){
            var self = this;
            var defer = $.Deferred();
            var store_id = self.ResUser[0].store_id[0];
            var date_min = moment().format('YYYY-MM-01');
            var date_max = moment().add(1, 'month').format('YYYY-MM-01');
            var fields = ['id','store_id', 'date', 'sale_objetive', 'purchase_limit', 'expense_limit', 'expected_profit'];
            var domain = [['date', '>=', date_min],['date', '<', date_max],['store_id','=',store_id]];
            var DashboardObjetive = new model.web.Model('dashboard.objetive');
            DashboardObjetive.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });
            return defer;
        },

        getResCurrency: function (id) {
            var self = this;
            return _.filter(self.ResCurrency,function (item) {
                return item.id === id;
            })
        },

        /* 
            ACCOUNT INVOICE
        */

        getTodayAccountInvoice:function() {
            var self = this;
            var date = moment().format('YYYY-MM-DD');
            return _.flatten(_.filter(self.AccountInvoice,function (inv) {
                return moment(inv.date_invoice).format('YYYY-MM-DD') === date;
            }));
        },

        getThisWeekAccountInvoice:function() {
            var self = this;
            var week = moment().week();
            return _.flatten(_.filter(self.AccountInvoice,function (inv) {
                return moment(inv.date_invoice).week() === week & moment(inv.date_invoice).format('YYYY')=== moment().format('YYYY');
            }));
        },

        getThisMonthAccountInvoice:function() {
            var self = this;
            return _.flatten(_.filter(self.AccountInvoice,function (inv) {
                return moment(inv.date_invoice).format('YYYY-MM')=== moment().format('YYYY-MM');
            }));
        },

        /* 
            ACCOUNT INVOICE BY MONTH
        */

        getMonthAccountInvoice:function(mes) {
            var self = this;
            if (mes < 10){
                var fecha = moment().format('YYYY')+'-'+'0'+mes;    
            }else{
                var fecha = moment().format('YYYY')+'-'+mes;    
            }
            return _.flatten(_.filter(self.AccountInvoice,function (inv) {
                return moment(inv.date_invoice).format('YYYY-MM') === fecha;
            }));
        },

        /*
            TODAY
        */

        showToday: function () {
            var self = this;
            var amount = 0;
            var data = [];
            var invoice = self.getTodayAccountInvoice();
            var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
            if(invoice.length > 0){
                _.each(invoice, function (item) {
                    var currency = self.getResCurrency(item.currency_id[0]).shift();
                    data.push({
                        amount: item.amount_total * (CurrencyBase.rate_silent / currency.rate_silent),
                    })
                });
                amount = _.reduce(_.map(data, function (map) {
                    return map.amount;
                }), function (memo, num) {
                    return memo + num;
                });
            }
            self.today = amount;
        },

        /*
            WEEK
        */

        showThisWeek: function () {
            var self = this;
            var amount = 0;
            var data = [];
            var invoice = self.getThisWeekAccountInvoice();
            var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
            if(invoice.length > 0){
                _.each(invoice, function (item) {
                    var currency = self.getResCurrency(item.currency_id[0]).shift();
                    data.push({
                        amount: item.amount_total * (CurrencyBase.rate_silent / currency.rate_silent),
                    })
                });
                amount = _.reduce(_.map(data, function (map) {
                    return map.amount;
                }), function (memo, num) {
                    return memo + num;
                });
            }
            self.week = amount;
        },

        /* 
            MONTH
        */

        showThisMonth: function () {
            var self = this;
            if(self.DashboardObjetive.length > 0){
                self.objective = self.DashboardObjetive.shift().purchase_limit;   
            }else{
                self.objective = 0;    
            }
            var amount = 0;
            var data = [];
            var invoice = self.getThisMonthAccountInvoice();
            var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
            if(invoice.length > 0){
                _.each(invoice, function (item) {
                    var currency = self.getResCurrency(item.currency_id[0]).shift();
                    data.push({
                        amount: item.amount_total * (CurrencyBase.rate_silent / currency.rate_silent),
                    })
                });
                amount = _.reduce(_.map(data, function (map) {
                    return map.amount;
                }), function (memo, num) {
                    return memo + num;
                });
            }
            self.month = amount;
            if(self.objective > 0){ 
                var percentage = (amount*100)/self.objective;    
            }else{
                var percentage = 0;
            }
            var chart2 = new ldBar("#circle-purchase",{
                'stroke':'#f9a825',
                'stroke-width':'10'
            });
            chart2.set(percentage);
            self.percentage = percentage;
            self.$el.find('.widget-content').find('a').text(accounting.formatMoney(amount, CurrencyBase.symbol, CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator))
        },

        showMonth: function() {
            var self = this;
            var order;
            var invoice;
            var title = ['Ene', 'Feb', 'Mar','Abr','May','Jun','Jul','Ago','Sep','Oct','Nov','Dic'];
            var data = [];
            var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
            for (var i = 1; i <= 12; i++) {
                var array = [];
                invoice = self.getMonthAccountInvoice(i);
                _.each(invoice, function (item) {
                    var currency = self.getResCurrency(item.currency_id[0]).shift();
                    array.push({
                        amount: item.amount_total * (CurrencyBase.rate_silent / currency.rate_silent),
                    })
                });
                var invoice_total = 0;
                if(array.length > 0){
                    invoice_total = _.reduce(_.map(array, function (map) {
                        return map.amount;
                    }), function (memo, num) {
                        return memo + num;
                    });
                }
                var total = invoice_total; 
                data.push(total);
            }
            self.data = data;
        },

        /*
            MODAL
        */
       
        showModal: function (e) {
            var self = this;
            self.showThisWeek();
            self.showToday();
            self.showMonth();
            var titleData = [
                {
                    title: "Compras"
                }
            ];
            var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
            var modal = Qweb.render('WidgetModal', {
                modalTitle: titleData,
                objective: accounting.formatMoney(self.objective, CurrencyBase.symbol, CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),
                today: accounting.formatMoney(self.today, CurrencyBase.symbol, CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),
                week: accounting.formatMoney(self.week, CurrencyBase.symbol, CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),
                month: accounting.formatMoney(self.month, CurrencyBase.symbol, CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator),
            });

            $('.openerp_webclient_container').after(modal);
            $('.widget-modal').modal()
            $('.widget-modal').on('hidden.bs.modal', function (e) {
                self.removeModal(e);
            })

            var chartX = new ldBar("#myItemModal",{
                'stroke':'#f9a825',
            });

            chartX.set(self.percentage);

            self.fetchChart(self.data);
        },

        removeModal: function (e) {
            $('.widget-modal').remove();
            $('.modal-backdrop').remove();
        },

        /* 
            BUILD A CHART
        */

        fetchChart: function (data) {
            var self = this;
            var label = ['Ene', 'Feb', 'Mar','Abr','May','Jun','Jul','Ago','Sep','Oct','Nov','Dic'];
            var body = data;

            Chart.scaleService.updateScaleDefaults('linear', {
              ticks: {
                callback: function(tick) {
                    return tick.toLocaleString('de-DE');
                }
              }
            });
            Chart.defaults.global.tooltips.callbacks.label = function(tooltipItem, data) {
                var dataset = data.datasets[tooltipItem.datasetIndex];
                var datasetLabel = dataset.label || '';
                return datasetLabel +  dataset.data[tooltipItem.index].toLocaleString('de-DE');
            };

            var chart = new Chart($("#prueba"), {
                type: 'line',
                data: {
                    labels: label,
                    datasets: [
                        {
                            label: false,
                            data: body,
                            backgroundColor: '#f9a825',
                            borderColor: '#f9a825',
                            borderWidth: 2,
                            fill: false,
                        }
                    ]
                },
                options: {
                    responsive: true,
                    title: {
                        display: false,
                    },
                    hover: {
                        mode: 'nearest',
                        intersect: true
                    },
                    legend: {
                       display: false,
                    },
                    layout: {
                        padding: {
                            top: 45,
                            bottom: 40,
                            left : 10,
                            rigth: 10,
                        }
                    },
                    elements: {
                        line: {
                            tension: 0.000001
                        }
                    },
                }
            });
        },

    });
}